@@ -12,7 +12,7 @@ export OPENSSL_ROOT_DIR=${OPENSSL_ROOT_DIR:-/usr/lib/x86_64-linux-gnu}
 export VERSION_SUFFIX=${VERSION_SUFFIX:stable}
 
 # Set defaults
-export SUFFIX=${DRONE_PULL_REQUEST:=master}
+export SUFFIX=${PR_ID:=${DRONE_PULL_REQUEST:=master}}
 if [ $SUFFIX != "master" ]; then
     SUFFIX="PR-$SUFFIX"
 fi
@@ -20,6 +20,11 @@ if [ "$BUILD_UPDATER" != "OFF" ]; then
     BUILD_UPDATER=ON
 fi
 
+# Ensure we use gcc-11 on RHEL-like systems
+if [ -e "/opt/rh/gcc-toolset-11/enable" ]; then
+    source /opt/rh/gcc-toolset-11/enable
+fi
+
 mkdir /app
 
 # Build client
@@ -42,11 +47,11 @@ DESTDIR=/app cmake --install .
 # Move stuff around
 cd /app
 
-mv usr/lib/x86_64-linux-gnu/* usr/lib/
+[ -d usr/lib/x86_64-linux-gnu ] && mv usr/lib/x86_64-linux-gnu/* usr/lib/
 
-mkdir usr/plugins
-mv usr/lib/*sync_vfs_suffix.so usr/plugins
-mv usr/lib/*sync_vfs_xattr.so usr/plugins
+mkdir -p AppDir/usr/plugins
+mv usr/lib64/*sync_vfs_suffix.so AppDir/usr/plugins || mv usr/lib/*sync_vfs_suffix.so AppDir/usr/plugins
+mv usr/lib64/*sync_vfs_xattr.so  AppDir/usr/plugins || mv usr/lib/*sync_vfs_xattr.so  AppDir/usr/plugins
 
 rm -rf usr/lib/cmake
 rm -rf usr/include
@@ -58,6 +63,10 @@ rm -rf usr/share/caja-python/
 rm -rf usr/share/nautilus-python/
 rm -rf usr/share/nemo-python/
 
+# The client-specific data dir also contains the translations, we want to have those in the AppImage.
+mkdir -p AppDir/usr/share
+mv usr/share/${EXECUTABLE_NAME} AppDir/usr/share/${EXECUTABLE_NAME}
+
 # Move sync exclude to right location
 mv /app/etc/*/sync-exclude.lst usr/bin/
 rm -rf etc
@@ -73,13 +82,12 @@ chmod a+x ${APPIMAGE_NAME}
 rm ./${APPIMAGE_NAME}
 cp -r ./squashfs-root ./linuxdeploy-squashfs-root
 
-export LD_LIBRARY_PATH=/app/usr/lib:${QT_BASE_DIR}/lib:/usr/local/lib/x86_64-linux-gnu:/usr/local/lib:/usr/local/lib64
+export LD_LIBRARY_PATH=/app/usr/lib64:/app/usr/lib:${QT_BASE_DIR}/lib:/usr/local/lib/x86_64-linux-gnu:/usr/local/lib:/usr/local/lib64
 ./linuxdeploy-squashfs-root/AppRun --desktop-file=${DESKTOP_FILE} --icon-file=usr/share/icons/hicolor/512x512/apps/Nextcloud.png --executable=usr/bin/${EXECUTABLE_NAME} --appdir=AppDir
 
 # Use linuxdeploy-plugin-qt to deploy qt dependencies
 export APPIMAGE_NAME=linuxdeploy-plugin-qt-x86_64.AppImage
-# TODO: this share link is supposed to be a workaround, we need an url pointing to github.com/linuxdeploy/linuxdeploy-plugin-qt/releases/tag/continuous, see github.com/linuxdeploy/linuxdeploy-plugin-qt/pull/186
-wget -O ${APPIMAGE_NAME} --ca-directory=/etc/ssl/certs -c "https://cloud.nextcloud.com/s/TxcGkDiSs2enKoN/download/linuxdeploy-plugin-qt-x86_64.AppImage"
+wget -O ${APPIMAGE_NAME} --ca-directory=/etc/ssl/certs -c "https://github.com/linuxdeploy/linuxdeploy-plugin-qt/releases/download/continuous/linuxdeploy-plugin-qt-x86_64.AppImage"
 chmod a+x ${APPIMAGE_NAME}
 ./${APPIMAGE_NAME} --appimage-extract
 rm ./${APPIMAGE_NAME}
@@ -91,11 +99,33 @@ export QML_SOURCES_PATHS=${DESKTOP_CLIENT_ROOT}/src/gui
 
 ./linuxdeploy-squashfs-root/AppRun --desktop-file=${DESKTOP_FILE} --icon-file=usr/share/icons/hicolor/512x512/apps/Nextcloud.png --executable=usr/bin/${EXECUTABLE_NAME} --appdir=AppDir --output appimage
 
+# Workaround issue #103 and #7231
+export APPIMAGETOOL=appimagetool-x86_64.AppImage
+wget -O ${APPIMAGETOOL} --ca-directory=/etc/ssl/certs -c https://github.com/AppImage/AppImageKit/releases/download/continuous/appimagetool-x86_64.AppImage
+chmod a+x ${APPIMAGETOOL}
+rm -rf ./squashfs-root
+./${APPIMAGETOOL} --appimage-extract
+rm ./${APPIMAGETOOL}
+cp -r ./squashfs-root ./appimagetool-squashfs-root
+rm -rf ./squashfs-root
+APPIMAGE=$(ls *.AppImage)
+./"${APPIMAGE}" --appimage-extract
+rm ./"${APPIMAGE}"
+rm ./squashfs-root/usr/lib/libglib-2.0.so.0
+LD_LIBRARY_PATH="$PWD/appimagetool-squashfs-root/usr/lib":$LD_LIBRARY_PATH PATH="$PWD/appimagetool-squashfs-root/usr/bin":$PATH appimagetool -n ./squashfs-root "${APPIMAGE}"
+
 #move AppImage
-if [ ! -z "$DRONE_COMMIT" ]
+export COMMIT=${GITHUB_SHA:=${DRONE_COMMIT}}
+if [ ! -z "$COMMIT" ]
 then
-    mv *.AppImage ${EXECUTABLE_NAME}-${SUFFIX}-${DRONE_COMMIT}-x86_64.AppImage
+    export APPIMAGE_NAME="${EXECUTABLE_NAME}-${SUFFIX}-${COMMIT}-x86_64.AppImage"
 else
-    mv *.AppImage ${EXECUTABLE_NAME}-${SUFFIX}-x86_64.AppImage
+    export APPIMAGE_NAME="${EXECUTABLE_NAME}-${SUFFIX}-x86_64.AppImage"
+fi
+mv *.AppImage ${DESKTOP_CLIENT_ROOT}/$APPIMAGE_NAME
+
+# tell GitHub Actions the name of our appimage
+if [ ! -z "$GITHUB_OUTPUT" ]; then
+  echo "AppImage name: ${APPIMAGE_NAME}"
+  echo "APPIMAGE_NAME=${APPIMAGE_NAME}" >> "$GITHUB_OUTPUT"
 fi
-mv *.AppImage ${DESKTOP_CLIENT_ROOT}/
