@@ -59,8 +59,9 @@ void PropagateRemoteMkdir::start()
     }
 
     _job = new DeleteJob(propagator()->account(),
-        propagator()->fullRemotePath(_item->_file),
-        this);
+                         propagator()->fullRemotePath(_item->_file),
+                         {},
+                         this);
     connect(qobject_cast<DeleteJob *>(_job), &DeleteJob::finishedSignal, this, &PropagateRemoteMkdir::slotMkdir);
     _job->start();
 }
@@ -261,6 +262,7 @@ void PropagateRemoteMkdir::slotEncryptFolderFinished(int status, EncryptionStatu
     qCDebug(lcPropagateRemoteMkdir) << "Success making the new folder encrypted";
     propagator()->_activeJobList.removeOne(this);
     _item->_e2eEncryptionStatus = encryptionStatus;
+    _item->_e2eCertificateFingerprint = propagator()->account()->encryptionCertificateFingerprint();
     _item->_e2eEncryptionStatusRemote = encryptionStatus;
     if (_item->isEncrypted()) {
         _item->_e2eEncryptionServerCapability = EncryptionStatusEnums::fromEndToEndEncryptionApiVersion(propagator()->account()->capabilities().clientSideEncryptionVersion());
